<?php

class ThriftClient extends CComponent
{
    public $server_host = 'localhost';
    public $server_port = '9090';
    public $send_timeout = 1;
    public $recv_timeout = 1;
    public $service_config = array();
    private $services = array();
    
    public function init()
    {
        //spl_autoload_register(array('ThriftClient','autoload'));
        require_once 'src/Thrift.php';
        require_once 'src/transport/TTransport.php';
        require_once 'src/transport/TSocket.php';
        require_once 'src/protocol/TBinaryProtocol.php';
        require_once 'src/transport/TFramedTransport.php';
        require_once 'src/transport/TBufferedTransport.php';
        $GLOBALS['THRIFT_ROOT'] = __DIR__ . "/src";
        Yii::import('application.vendors.thrift.src.*');

        $svcs = scandir($GLOBALS['THRIFT_ROOT'] . '/packages');
        foreach ($svcs as $svc) {
            if ($svc == '.' || $svc == '..') continue;
            $sdir = $GLOBALS['THRIFT_ROOT'] . '/packages/' . $svc;
            $files = scandir($sdir);
            foreach ($files as $f) {
                if ($f == '.' || $f == '..' || $f == $svc . '_types.php') 
                    continue;
                if ($this->endwith($f, '.php')) {
                    include $GLOBALS['THRIFT_ROOT'] . '/packages/' . $svc . '/' . $f;
                    $class = substr($f, 0, strlen($f) - 4) . 'Client';
                    $this->services[$svc] = $class;
                    break;
                }
            }
        }
    }
        
    public function __get($name) {
        if (isset($this->services[$name])) {
            if (is_string($this->services[$name])) {
                if (isset($this->service_config[$name])) {
                    $config = $this->service_config[$name];
                    if (empty($config['send_timeout'])) $config['send_timeout'] = $this->send_timeout;
                    if (empty($config['recv_timeout'])) $config['recv_timeout'] = $this->recv_timeout;
                    $transport = new TSocket($config['server_host'], $config['server_port']);
                    $transport->setSendTimeout($config['send_timeout'] * 1000);
                    $transport->setRecvTimeout($config['recv_timeout'] * 1000);
                    $transport->open();
                    $protocol = new TBinaryProtocol(new TBufferedTransport($transport));
                    $class = $this->services[$name];
                    $this->services[$name] = new $class($protocol);
                } else {
                    $transport = new TSocket($this->server_host, $this->server_port);
                    $transport->setSendTimeout($this->send_timeout * 1000);
                    $transport->setRecvTimeout($this->recv_timeout * 1000);
                    $transport->open();
                    $protocol = new TBinaryProtocol(new TBufferedTransport($transport));
                    $class = $this->services[$name];
                    $this->services[$name] = new $class($protocol);
                }
            }
            return $this->services[$name];
        } else {
            throw new Exception('Service Not Defined');
        }
    }
   
    private function endwith($s1, $s2)
    {
        return substr($s1, strlen($s1) - strlen($s2)) == $s2;
    }
}
